@@ -50,8 +50,9 @@ setMethod("show",
 ##                    cat('\t\t',n,' = ',deparse(object@pars[[n]]),'\n')
                         k <- object@pars[[n]]
                         k <- if (is.list(k)) paste(paste(names(k),k,sep='='),collapse=" ") else paste(k,collapse=' ')
-                        k <- if (nchar(k) > 20) paste(substr(k,1,20),' ...') else k
-                        cat('\t\t',n,' -> ',k,'\n')
+                        cat('\t\t',n,' -> ')
+                        if (nchar(k) > 50) cat(unlist(strsplit(k," ")), fill=50) else cat(k,"\n")
+                        #cat("\n")
                     }
                 }
             }
@@ -104,9 +105,9 @@ setMethod("show",
           function(object) {
             userSplit <- !is.null(object@dataSplits)
             cat(ifelse(!userSplit & object@strat,'Stratified ',''),
-                object@nReps,'x')
+                object@nReps,'x ')
             
-            if (object@hldSz < 1) cat(100*(1-object@hldSz),'%/',100*object@hldSz,'% Holdout\n')
+            if (object@hldSz < 1) cat(100*(1-object@hldSz),'% /',100*object@hldSz,'% Holdout\n')
             else cat(object@hldSz," cases Holdout\n")
             
             if (!userSplit)
@@ -415,7 +416,7 @@ setMethod("show",
 # Luis Torgo, Aug 2009
 # =====================================================
 # Example runs:
-# > plot(subset(nnet,stats='e1',vars=1:4))
+# > plot(subset(nnet,metrics='e1',vars=1:4))
 #
 setMethod("subset",
           signature(x='ComparisonResults'),
